Type: Package
Package: thisutils
Title: Collection of Utility Functions for Data Analysis and Computing
Version: 0.4.7
Date: 2026-05-24
Authors@R: 
    c(
    person("Meng", "Xu", email = "mengxu98@qq.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-8300-1054")),
    person(given = "Haoliang", family = "Zhu", email = "hurry060215@gmail.com", role = c("aut"), comment = c(ORCID = "0009-0004-9766-5179"))
    )
Maintainer: Meng Xu <mengxu98@qq.com>
Description: Provides utility functions for data analysis and computing. Includes functions for logging, parallel processing, and other computational tasks to streamline workflows.
License: MIT + file LICENSE
URL: https://mengxu98.github.io/thisutils/
BugReports: https://github.com/mengxu98/thisutils/issues
Depends: R (>= 4.1.0)
Imports: cli, doParallel, foreach, Matrix, pak, parallel, Rcpp, rlang,
        stats, utils
Suggests: testthat (>= 3.0.0)
LinkingTo: Rcpp
Config/Needs/website: mengxu98/thistemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-05-24 13:33:26 UTC; mx
Author: Meng Xu [aut, cre] (ORCID: <https://orcid.org/0000-0002-8300-1054>),
  Haoliang Zhu [aut] (ORCID: <https://orcid.org/0009-0004-9766-5179>)
Repository: CRAN
Date/Publication: 2026-05-24 14:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-24 14:07:49 UTC; unix
Archs: thisutils.so.dSYM
