# Generated by roxygen2: do not edit by hand

S3method(as.character,figlet_text)
S3method(print,figlet_font)
S3method(print,figlet_text)
S3method(print,thisutils_logo)
export("%ss%")
export(add_pkg_file)
export(as_matrix)
export(capitalize)
export(check_ci_env)
export(check_pkg_status)
export(check_r)
export(check_sparsity)
export(classification_metrics_compute)
export(collapse_sparse_rows)
export(compute_lisi)
export(download)
export(fast_row_vars)
export(figlet)
export(figlet_font)
export(filter_nonzero_variance_features)
export(get_namespace_fun)
export(get_verbose)
export(invoke_fun)
export(is_apple_silicon)
export(is_linux)
export(is_osx)
export(is_outlier)
export(is_windows)
export(log_message)
export(matrix_process)
export(matrix_to_table)
export(max_depth)
export(maximump)
export(meanp)
export(minimump)
export(normalization)
export(parallelize_fun)
export(parse_inline_expressions)
export(pearson_correlation)
export(r_square)
export(remove_r)
export(remove_space)
export(run_biocneighbors_knn)
export(run_dense_topk_by_column)
export(run_sparse_topk_by_column)
export(simulate_sparse_matrix)
export(sparse_cor)
export(split_indices)
export(sump)
export(table_to_matrix)
export(thisutils_logo)
export(try_get)
export(unnest_fun)
export(votep)
export(wilkinsonp)
importFrom(Rcpp,sourceCpp)
useDynLib(thisutils, .registration = TRUE)
