# **thisutils** <img src="man/figures/logo.svg" align="right" width="120"/>

<!-- badges: start -->

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/thisutils)](https://CRAN.R-project.org/package=thisutils) [![Conda Version](https://img.shields.io/conda/vn/conda-forge/r-thisutils?color=green)](https://anaconda.org/conda-forge/r-thisutils) [![version](https://img.shields.io/github/r-package/v/mengxu98/thisutils?label=version&color=blue)](https://github.com/mengxu98/thisutils/blob/main/DESCRIPTION) [![R-CMD-check](https://github.com/mengxu98/thisutils/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/mengxu98/thisutils/actions/workflows/R-CMD-check.yaml) [![test-coverage](https://github.com/mengxu98/thisutils/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/mengxu98/thisutils/actions/workflows/test-coverage.yaml) [![pkgdown](https://github.com/mengxu98/thisutils/actions/workflows/pkgdown.yaml/badge.svg)](https://mengxu98.github.io/thisutils/reference/index.html) [![RStudio CRAN mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/thisutils)](https://CRAN.R-project.org/package=thisutils)

<!-- badges: end -->

## **Introduction**

[thisutils](https://mengxu98.github.io/thisutils/) is an R package for providing utility functions for data analysis and computing. Includes functions for logging, parallel processing, and other computational tasks to streamline workflows.

## **Installation**

Install CRAN version:

``` r
install.packages("thisutils")
# or
if (!require("pak", quietly = TRUE)) {
  install.packages("pak")
}
pak::pak("thisutils")
```

Install development version from [GitHub](https://github.com/mengxu98/thisutils) use [pak](https://github.com/r-lib/pak):

``` r
if (!require("pak", quietly = TRUE)) {
  install.packages("pak")
}
pak::pak("mengxu98/thisutils")
```

## **Usage**

Please reference [here](https://mengxu98.github.io/thisutils/reference/index.html).
