% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_pkg_file.R
\name{add_pkg_file}
\alias{add_pkg_file}
\title{Add a package file and print package information}
\usage{
add_pkg_file(
  use_figlet = TRUE,
  figlet_font = "Slant",
  colors = c("red", "yellow", "green", "magenta", "cyan", "yellow", "green", "white",
    "magenta", "cyan"),
  verbose = TRUE
)
}
\arguments{
\item{use_figlet}{Whether to use figlet for ASCII art generation.
Default is \code{TRUE}. Details see \link{figlet}.}

\item{figlet_font}{Character string, figlet font to use.
Default is \code{"Slant"}.}

\item{colors}{Character vector, colors to use for the logo elements.}

\item{verbose}{Whether to print the message.
Default is \code{TRUE}.}
}
\value{
Creates a file named \verb{R/<pkg_name>-package.R}.
}
\description{
Add a package file and print package information
}
