% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thisutils-package.R
\name{print.thisutils_logo}
\alias{print.thisutils_logo}
\title{Print logo}
\usage{
\method{print}{thisutils_logo}(x, ...)
}
\arguments{
\item{x}{Input information.}

\item{...}{Other parameters.}
}
\value{
Print the ASCII logo
}
\description{
Print logo
}
