% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_linux}
\alias{is_linux}
\title{Check if the operating system is Linux}
\usage{
is_linux()
}
\value{
A logical value.
}
\description{
Check if the operating system is Linux
}
