% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_ci_env}
\alias{check_ci_env}
\title{Check CI environment}
\usage{
check_ci_env()
}
\value{
A logical value.
}
\description{
Check CI environment
}
