% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computation.R
\name{check_sparsity}
\alias{check_sparsity}
\title{Check sparsity of matrix}
\usage{
check_sparsity(x)
}
\arguments{
\item{x}{A matrix.}
}
\value{
Sparsity of matrix.
}
\description{
Check sparsity of matrix
}
