% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sump}
\alias{sump}
\title{Sum P-value}
\usage{
sump(p)
}
\arguments{
\item{p}{A vector of P-values.}
}
\description{
Sum P-value
}
\examples{
p <- c(0.01, 0.02, 0.03, 0.04, 0.05)
sump(p)
}
