% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computation.R
\name{r_square}
\alias{r_square}
\title{Coefficient of determination (\eqn{R^2})}
\usage{
r_square(y_true, y_pred)
}
\arguments{
\item{y_true}{A numeric vector with ground truth values.}

\item{y_pred}{A numeric vector with predicted values.}
}
\value{
The \eqn{R^2} value.
}
\description{
Coefficient of determination (\eqn{R^2})
}
\examples{
y <- rnorm(100)
y_pred <- y + rnorm(100, sd = 0.5)
r_square(y, y_pred)
}
